@@ -2,7 +2,7 @@
     zpoller - trivial socket poller class
 
     -------------------------------------------------------------------------
-    Copyright (c) 1991-2013 iMatix Corporation <www.imatix.com>
+    Copyright (c) 1991-2014 iMatix Corporation <www.imatix.com>
     Copyright other contributors as noted in the AUTHORS file.
 
     This file is part of CZMQ, the high-level C binding for 0MQ:
@@ -42,10 +42,17 @@ CZMQ_EXPORT zpoller_t *
 CZMQ_EXPORT void
     zpoller_destroy (zpoller_t **self_p);
 
+// Add a reader to be polled.
+CZMQ_EXPORT int
+    zpoller_add (zpoller_t *self, void *reader);
+
 //  Poll the registered readers for I/O, return first socket that has input.
 //  This means the order that sockets are defined in the poll list affects
 //  their priority. If you need a balanced poll, use the low level zmq_poll
-//  method directly.
+//  method directly. If the poll call was interrupted (SIGINT), or the ZMQ
+//  context was destroyed, or the timeout expired, returns NULL. You can
+//  test the actual exit condition by calling zpoller_expired () and
+//  zpoller_terminated (). Timeout is in msec.
 CZMQ_EXPORT void *
     zpoller_wait (zpoller_t *self, int timeout);
 
