@@ -4,7 +4,7 @@
     Runs all selftests.
 
     -------------------------------------------------------------------------
-    Copyright (c) 1991-2013 iMatix Corporation <www.imatix.com>
+    Copyright (c) 1991-2014 iMatix Corporation <www.imatix.com>
     Copyright other contributors as noted in the AUTHORS file.
 
     This file is part of czmq, the high-level C binding for 0MQ:
@@ -39,15 +39,20 @@ int main (int argc, char *argv [])
     printf ("Running CZMQ selftests...\n");
 
     //  These are ordered from most basic to most complex
-    zconfig_test (verbose);
+    zrex_test (verbose);
     zsys_test (verbose);
+    zchunk_test (verbose);
+    zconfig_test (verbose);
     zmutex_test (verbose);
     zclock_test (verbose);
-    zchunk_test (verbose);
+    zdir_patch_test (verbose);
+    zdir_test (verbose);
+    zdigest_test (verbose);
     zfile_test (verbose);
     zhash_test (verbose);
     ztree_test (verbose);
     zlist_test (verbose);
+    zuuid_test (verbose);
     zctx_test (verbose);
     zsockopt_test (verbose);
     zsocket_test (verbose);
@@ -57,10 +62,14 @@ int main (int argc, char *argv [])
     zstr_test (verbose);
     zmsg_test (verbose);
     zloop_test (verbose);
+    zproxy_test (verbose);
     zbeacon_test (verbose);
     zcert_test (verbose);
     zcertstore_test (verbose);
     zauth_test (verbose);
+#if (ZMQ_VERSION_MAJOR == 4)
+//     zmonitor_test (verbose);
+#endif
     printf ("Tests passed OK\n");
     return 0;
 }
