@@ -2,7 +2,7 @@
     zauth - authentication for ZeroMQ security mechanisms
 
     -------------------------------------------------------------------------
-    Copyright (c) 1991-2013 iMatix Corporation <www.imatix.com>
+    Copyright (c) 1991-2014 iMatix Corporation <www.imatix.com>
     Copyright other contributors as noted in the AUTHORS file.
 
     This file is part of CZMQ, the high-level C binding for 0MQ:
@@ -35,6 +35,7 @@
 */
 
 #include "../include/czmq.h"
+#include "platform.h"
 
 //  Structure of our class
 //  All work is done by a background thread, the "agent", which we talk
@@ -57,7 +58,7 @@ static void
 //  Install authentication for the specified context. Returns a new zauth
 //  object that you can use to configure authentication. Note that until you
 //  add policies, all incoming NULL connections are allowed (classic ZeroMQ
-//  behaviour), and all PLAIN and CURVE connections are denied. If there was
+//  behavior), and all PLAIN and CURVE connections are denied. If there was
 //  an error during initialization, returns NULL.
 
 zauth_t *
@@ -65,100 +66,24 @@ zauth_new (zctx_t *ctx)
 {    
     zauth_t *self = (zauth_t *) zmalloc (sizeof (zauth_t));
     assert (self);
-    
+
     //  Start background agent and wait for it to initialize
+    assert (ctx);
     self->pipe = zthread_fork (ctx, s_agent_task, NULL);
-    char *status = zstr_recv (self->pipe);
-    if (strneq (status, "OK"))
-        zauth_destroy (&self);
-    free (status);
+    if (self->pipe) {
+        char *status = zstr_recv (self->pipe);
+        if (strneq (status, "OK"))
+            zauth_destroy (&self);
+        zstr_free (&status);
+    }
+    else {
+        free (self);
+        self = NULL;
+    }
     return self;
 }
 
 
-//  --------------------------------------------------------------------------
-//  Allow (whitelist) a single IP address. For NULL, all clients from this
-//  address will be accepted. For PLAIN and CURVE, they will be allowed to
-//  continue with authentication. You can call this method multiple times 
-//  to whitelist multiple IP addresses. If you whitelist a single address,
-//  any non-whitelisted addresses are treated as blacklisted.
-
-void
-zauth_allow (zauth_t *self, char *address)
-{
-    zstr_sendx (self->pipe, "ALLOW", address, NULL);
-}
-
-
-//  --------------------------------------------------------------------------
-//  Deny (blacklist) a single IP address. For all security mechanisms, this
-//  rejects the connection without any further authentication. Use either a
-//  whitelist, or a blacklist, not not both. If you define both a whitelist 
-//  and a blacklist, only the whitelist takes effect.
-
-void
-zauth_deny (zauth_t *self, char *address)
-{
-    zstr_sendx (self->pipe, "DENY", address, NULL);
-}
-
-
-//  --------------------------------------------------------------------------
-//  Configure PLAIN authentication for a given domain. PLAIN authentication
-//  uses a plain-text password file. The filename is treated as a printf 
-//  format. To cover all domains, use "*". You can modify the password file
-//  at any time; it is reloaded automatically.
-
-void
-zauth_configure_plain (zauth_t *self, char *domain, char *filename, ...)
-{
-    assert (self);
-    assert (domain);
-    va_list argptr;
-    va_start (argptr, filename);
-    char *formatted = zsys_vprintf (filename, argptr);
-    va_end (argptr);
-    zstr_sendx (self->pipe, "PLAIN", domain, formatted, NULL);
-    free (formatted);
-}
-
-
-//  --------------------------------------------------------------------------
-//  Configure CURVE authentication for a given domain. CURVE authentication
-//  uses a directory that holds all public client certificates, i.e. their
-//  public keys. The certificates must be in zcert_save () format. The 
-//  location is treated as a printf format. To cover all domains, use "*". 
-//  You can add and remove certificates in that directory at any time. 
-//  To allow all client keys without checking, specify CURVE_ALLOW_ANY for
-//  the location.
-
-void
-zauth_configure_curve (zauth_t *self, char *domain, char *location, ...)
-{
-    assert (self);
-    assert (domain);
-    assert (location);
-    va_list argptr;
-    va_start (argptr, location);
-    char *formatted = zsys_vprintf (location, argptr);
-    va_end (argptr);
-    zstr_sendx (self->pipe, "CURVE", domain, formatted, NULL);
-    free (formatted);
-}
-
-
-//  --------------------------------------------------------------------------
-//  Enable verbose tracing of commands and activity
-
-void
-zauth_set_verbose (zauth_t *self, bool verbose)
-{
-    assert (self);
-    zstr_sendm (self->pipe, "VERBOSE");
-    zstr_send  (self->pipe, "%d", verbose);
-}
-
-
 //  --------------------------------------------------------------------------
 //  Destructor
 
@@ -169,13 +94,96 @@ zauth_destroy (zauth_t **self_p)
     if (*self_p) {
         zauth_t *self = *self_p;
         zstr_send (self->pipe, "TERMINATE");
-        free (zstr_recv (self->pipe));
+        char *reply = zstr_recv (self->pipe);
+        zstr_free (&reply);
         free (self);
         *self_p = NULL;
     }
 }
 
 
+//  --------------------------------------------------------------------------
+//  Allow (whitelist) a single IP address. For NULL, all clients from this
+//  address will be accepted. For PLAIN and CURVE, they will be allowed to
+//  continue with authentication. You can call this method multiple times 
+//  to whitelist multiple IP addresses. If you whitelist a single address,
+//  any non-whitelisted addresses are treated as blacklisted.
+
+void
+zauth_allow (zauth_t *self, const char *address)
+{
+    zstr_sendx (self->pipe, "ALLOW", address, NULL);
+    //  Wait for completion
+    free (zstr_recv (self->pipe));
+}
+
+
+//  --------------------------------------------------------------------------
+//  Deny (blacklist) a single IP address. For all security mechanisms, this
+//  rejects the connection without any further authentication. Use either a
+//  whitelist, or a blacklist, not not both. If you define both a whitelist 
+//  and a blacklist, only the whitelist takes effect.
+
+void
+zauth_deny (zauth_t *self, const char *address)
+{
+    zstr_sendx (self->pipe, "DENY", address, NULL);
+    //  Wait for completion
+    free (zstr_recv (self->pipe));
+}
+
+
+//  --------------------------------------------------------------------------
+//  Configure PLAIN authentication for a given domain. PLAIN authentication
+//  uses a plain-text password file. To cover all domains, use "*". You can
+//  modify the password file at any time; it is reloaded automatically.
+
+void
+zauth_configure_plain (zauth_t *self, const char *domain, const char *filename)
+{
+    assert (self);
+    assert (domain);
+    assert (filename);
+    zstr_sendx (self->pipe, "PLAIN", domain, filename, NULL);
+    //  Wait for completion
+    free (zstr_recv (self->pipe));
+}
+
+
+//  --------------------------------------------------------------------------
+//  Configure CURVE authentication for a given domain. CURVE authentication
+//  uses a directory that holds all public client certificates, i.e. their
+//  public keys. The certificates must be in zcert_save () format. To cover
+//  all domains, use "*". You can add and remove certificates in that
+//  directory at any time. To allow all client keys without checking, specify
+//  CURVE_ALLOW_ANY for the location.
+
+void
+zauth_configure_curve (zauth_t *self, const char *domain, const char *location)
+{
+    assert (self);
+    assert (domain);
+    assert (location);
+    zstr_sendx (self->pipe, "CURVE", domain, location, NULL);
+    //  Wait for completion
+    free (zstr_recv (self->pipe));
+}
+
+
+//  --------------------------------------------------------------------------
+//  Enable verbose tracing of commands and activity
+
+void
+zauth_set_verbose (zauth_t *self, bool verbose)
+{
+    assert (self);
+    zstr_sendm (self->pipe, "VERBOSE");
+    zstr_sendf (self->pipe, "%d", verbose);
+    //  Wait for completion
+    free (zstr_recv (self->pipe));
+}
+
+
 //  *************************     ZAP REQUESTS     *************************
 
 //  A small class for working with ZAP requests and replies
@@ -312,7 +320,8 @@ s_agent_new (zctx_t *ctx, void *pipe)
 
     //  Create ZAP handler and get ready for requests
     self->handler = zsocket_new (self->ctx, ZMQ_REP);
-    if (zsocket_bind (self->handler, "inproc://zeromq.zap.01") == 0)
+    if (self->handler
+    &&  zsocket_bind (self->handler, "inproc://zeromq.zap.01") == 0)
         zstr_send (self->pipe, "OK");
     else
         zstr_send (self->pipe, "ERROR");
@@ -349,32 +358,35 @@ s_agent_handle_pipe (agent_t *self)
     if (streq (command, "ALLOW")) {
         char *address = zmsg_popstr (request);
         zhash_insert (self->whitelist, address, "OK");
-        free (address);
+        zstr_free (&address);
+        zstr_send (self->pipe, "OK");
     }
     else
     if (streq (command, "DENY")) {
         char *address = zmsg_popstr (request);
         zhash_insert (self->blacklist, address, "OK");
-        free (address);
+        zstr_free (&address);
+        zstr_send (self->pipe, "OK");
     }
     else
     if (streq (command, "PLAIN")) {
         //  For now we don't do anything with domains
         char *domain = zmsg_popstr (request);
-        free (domain);
+        zstr_free (&domain);
         //  Get password file and load into zhash table
         //  If the file doesn't exist we'll get an empty table
         char *filename = zmsg_popstr (request);
         zhash_destroy (&self->passwords);
         self->passwords = zhash_new ();
         zhash_load (self->passwords, filename);
-        free (filename);
+        zstr_free (&filename);
+        zstr_send (self->pipe, "OK");
     }
     else
     if (streq (command, "CURVE")) {
         //  For now we don't do anything with domains
         char *domain = zmsg_popstr (request);
-        free (domain);
+        zstr_free (&domain);
         //  If location is CURVE_ALLOW_ANY, allow all clients. Otherwise 
         //  treat location as a directory that holds the certificates.
         char *location = zmsg_popstr (request);
@@ -385,13 +397,15 @@ s_agent_handle_pipe (agent_t *self)
             self->certstore = zcertstore_new (location);
             self->allow_any = false;
         }
-        free (location);
+        zstr_free (&location);
+        zstr_send (self->pipe, "OK");
     }
     else
     if (streq (command, "VERBOSE")) {
         char *verbose = zmsg_popstr (request);
         self->verbose = *verbose == '1';
-        free (verbose);
+        zstr_free (&verbose);
+        zstr_send (self->pipe, "OK");
     }
     else
     if (streq (command, "TERMINATE")) {
@@ -402,7 +416,7 @@ s_agent_handle_pipe (agent_t *self)
         printf ("E: invalid command from API: %s\n", command);
         assert (false);
     }
-    free (command);
+    zstr_free (&command);
     zmsg_destroy (&request);
     return 0;
 }
@@ -426,12 +440,12 @@ s_agent_authenticate (agent_t *self)
             if (zhash_lookup (self->whitelist, request->address)) {
                 allowed = true;
                 if (self->verbose) 
-                    printf ("I: PASSED (whitelist) address=%s\n", request->address);
+                    printf ("ZAUTH I: PASSED (whitelist) address=%s\n", request->address);
             }
             else {
                 denied = true;
                 if (self->verbose) 
-                    printf ("I: DENIED (not in whitelist) address=%s\n", request->address);
+                    printf ("ZAUTH I: DENIED (not in whitelist) address=%s\n", request->address);
             }
         }
         else
@@ -439,12 +453,12 @@ s_agent_authenticate (agent_t *self)
             if (zhash_lookup (self->blacklist, request->address)) {
                 denied = true;
                 if (self->verbose) 
-                    printf ("I: DENIED (blacklist) address=%s\n", request->address);
+                    printf ("ZAUTH I: DENIED (blacklist) address=%s\n", request->address);
             }
             else {
                 allowed = true;
                 if (self->verbose) 
-                    printf ("I: PASSED (not in blacklist) address=%s\n", request->address);
+                    printf ("ZAUTH I: PASSED (not in blacklist) address=%s\n", request->address);
             }
         }
         //  Mechanism-specific checks
@@ -452,7 +466,7 @@ s_agent_authenticate (agent_t *self)
             if (streq (request->mechanism, "NULL") && !allowed) {
                 //  For NULL, we allow if the address wasn't blacklisted
                 if (self->verbose) 
-                    printf ("I: ALLOWED (NULL)\n");
+                    printf ("ZAUTH I: ALLOWED (NULL)\n");
                 allowed = true;
             }
             else
@@ -485,20 +499,20 @@ s_authenticate_plain (agent_t *self, zap_request_t *request)
         char *password = (char *) zhash_lookup (self->passwords, request->username);
         if (password && streq (password, request->password)) {
             if (self->verbose) 
-                printf ("I: ALLOWED (PLAIN) username=%s password=%s\n",
+                printf ("ZAUTH I: ALLOWED (PLAIN) username=%s password=%s\n",
                     request->username, request->password);
             return true;
         }
         else {
             if (self->verbose) 
-                printf ("I: DENIED (PLAIN) username=%s password=%s\n",
+                printf ("ZAUTH I: DENIED (PLAIN) username=%s password=%s\n",
                     request->username, request->password);
             return false;
         }
     }
     else {
         if (self->verbose) 
-            printf ("I: DENIED (PLAIN) no password file defined\n");
+            printf ("ZAUTH I: DENIED (PLAIN) no password file defined\n");
         return false;
     }
 }
@@ -510,19 +524,19 @@ s_authenticate_curve (agent_t *self, zap_request_t *request)
     //  TODO: load metadata from certificate and return via ZAP response
     if (self->allow_any) {
         if (self->verbose) 
-            printf ("I: ALLOWED (CURVE allow any client)\n");
+            printf ("ZAUTH I: ALLOWED (CURVE allow any client)\n");
         return true;
     }
     else
     if (self->certstore
     &&  zcertstore_lookup (self->certstore, request->client_key)) {
         if (self->verbose) 
-            printf ("I: ALLOWED (CURVE) client_key=%s\n", request->client_key);
+            printf ("ZAUTH I: ALLOWED (CURVE) client_key=%s\n", request->client_key);
         return true;
     }
     else {
         if (self->verbose) 
-            printf ("I: DENIED (CURVE) client_key=%s\n", request->client_key);
+            printf ("ZAUTH I: DENIED (CURVE) client_key=%s\n", request->client_key);
         return false;
     }
 }
@@ -544,9 +558,8 @@ s_agent_task (void *args, zctx_t *ctx, void *pipe)
         if (which == self->pipe)
             s_agent_handle_pipe (self);
         else
-        if (which == self->handler) {
+        if (which == self->handler)
             s_agent_authenticate (self);
-        }
     }
     //  Done, free all agent resources
     zpoller_destroy (&poller);
@@ -558,32 +571,22 @@ s_agent_task (void *args, zctx_t *ctx, void *pipe)
 //  Selftest
 
 #if (ZMQ_VERSION_MAJOR == 4)
-//  Checks whether client can connect to server, and if so, returns
-//  true; else returns false after a short timeout.
-
+//  Checks whether client can connect to server
 static bool
-s_can_connect (void *server, void *client)
+s_can_connect (zctx_t *ctx, void **server, void **client)
 {
-    //  We'll do each test on a new port number since otherwise we have to
-    //  destroy and recreate the sockets each time.
-    static int port_nbr = 9000;
-    int rc = zsocket_bind (server, "tcp://*:%d", port_nbr);
-    assert (rc == port_nbr);
-    rc = zsocket_connect (client, "tcp://localhost:%d", port_nbr);
+    int port_nbr = zsocket_bind (*server, "tcp://127.0.0.1:*");
+    assert (port_nbr > 0);
+    int rc = zsocket_connect (*client, "tcp://127.0.0.1:%d", port_nbr);
     assert (rc == 0);
-    
-    zpoller_t *poller = zpoller_new (client, NULL);
-    zstr_send (server, "Hello, World");
-    //  Need up to half a second if running under Valgrind
-    bool success = zpoller_wait (poller, 500) == client;
-    if (success)
-        free (zstr_recv (client));
+    zstr_send (*server, "Hello, World");
+    zpoller_t *poller = zpoller_new (*client, NULL);
+    bool success = (zpoller_wait (poller, 200) == *client);
     zpoller_destroy (&poller);
-    rc = zsocket_unbind (server, "tcp://*:%d", port_nbr);
-    assert (rc != -1);
-    rc = zsocket_disconnect (client, "tcp://localhost:%d", port_nbr);
-    assert (rc != -1);
-    port_nbr++;
+    zsocket_destroy (ctx, *client);
+    zsocket_destroy (ctx, *server);
+    *server = zsocket_new (ctx, ZMQ_PUSH);
+    *client = zsocket_new (ctx, ZMQ_PULL);
     return success;
 }
 #endif
@@ -608,96 +611,99 @@ zauth_test (bool verbose)
     assert (auth);
     zauth_set_verbose (auth, verbose);
     
-    //  A default NULL connection should always success, and not go through
-    //  our authentication infrastructure at all.
+    //  A default NULL connection should always success, and not 
+    //  go through our authentication infrastructure at all.
     void *server = zsocket_new (ctx, ZMQ_PUSH);
     void *client = zsocket_new (ctx, ZMQ_PULL);
-    bool success = s_can_connect (server, client);
+    bool success = s_can_connect (ctx, &server, &client);
     assert (success);
     
-    //  When we set a domain on the server, we switch on authentication 
-    //  for NULL sockets, but with no policies, the client connection will 
-    //  be allowed.
-    //
-    //  TODO: libzmq should accept new security options after unbind/bind
-    //  but for now we have to create a new server socket each time.
-    server = zsocket_new (ctx, ZMQ_PUSH);
+    //  When we set a domain on the server, we switch on authentication
+    //  for NULL sockets, but with no policies, the client connection 
+    //  will be allowed.
     zsocket_set_zap_domain (server, "global");
-    success = s_can_connect (server, client);
+    success = s_can_connect (ctx, &server, &client);
     assert (success);
-        
-    //  Blacklist 127.0.0.1, connection should fail
-    zauth_deny (auth, "127.0.0.1");
-    success = s_can_connect (server, client);
-    assert (!success);
     
+    //  Blacklist 127.0.0.1, connection should fail
+    zsocket_set_zap_domain (server, "global");
+    zauth_deny (auth, "127.0.0.1");
+    success = s_can_connect (ctx, &server, &client);
+    assert (!success);
+
     //  Whitelist our address, which overrides the blacklist
+    zsocket_set_zap_domain (server, "global");
     zauth_allow (auth, "127.0.0.1");
-    success = s_can_connect (server, client);
+    success = s_can_connect (ctx, &server, &client);
     assert (success);
 
     //  Try PLAIN authentication
+    zsocket_set_plain_server (server, 1);
+    zsocket_set_plain_username (client, "admin");
+    zsocket_set_plain_password (client, "Password");
+    success = s_can_connect (ctx, &server, &client);
+    assert (!success);
+    
     FILE *password = fopen (TESTDIR "/password-file", "w");
     assert (password);
     fprintf (password, "admin=Password\n");
     fclose (password);
-    
     zsocket_set_plain_server (server, 1);
     zsocket_set_plain_username (client, "admin");
     zsocket_set_plain_password (client, "Password");
-    success = s_can_connect (server, client);
-    assert (!success);
-
     zauth_configure_plain (auth, "*", TESTDIR "/password-file");
-    success = s_can_connect (server, client);
+    success = s_can_connect (ctx, &server, &client);
     assert (success);
 
+    zsocket_set_plain_server (server, 1);
+    zsocket_set_plain_username (client, "admin");
     zsocket_set_plain_password (client, "Bogus");
-    success = s_can_connect (server, client);
+    success = s_can_connect (ctx, &server, &client);
     assert (!success);
 
 #   if defined (HAVE_LIBSODIUM)
     //  Try CURVE authentication
-    //  We'll create two new certificates and save the client public 
+    //  We'll create two new certificates and save the client public
     //  certificate on disk; in a real case we'd transfer this securely
     //  from the client machine to the server machine.
     zcert_t *server_cert = zcert_new ();
-    zcert_apply (server_cert, server);
-    zsocket_set_curve_server (server, 1);
-
     zcert_t *client_cert = zcert_new ();
-    zcert_apply (client_cert, client);
     char *server_key = zcert_public_txt (server_cert);
+    
+    //  Test without setting-up any authentication
+    zcert_apply (server_cert, server);
+    zcert_apply (client_cert, client);
+    zsocket_set_curve_server (server, 1);
     zsocket_set_curve_serverkey (client, server_key);
-
-    //  We've not set-up any authentication, connection will fail
-    success = s_can_connect (server, client);
+    success = s_can_connect (ctx, &server, &client);
     assert (!success);
 
-    //  PH: 2013/09/18
-    //  There's an issue with libzmq where it sometimes fails to 
-    //  connect even if the ZAP handler allows it. It's timing
-    //  dependent, so this is a voodoo hack. To be removed, I've
-    //  no idea this even applies to all boxes.
-    sleep (1);
-
     //  Test CURVE_ALLOW_ANY
+    zcert_apply (server_cert, server);
+    zcert_apply (client_cert, client);
+    zsocket_set_curve_server (server, 1);
+    zsocket_set_curve_serverkey (client, server_key);
     zauth_configure_curve (auth, "*", CURVE_ALLOW_ANY);
-    success = s_can_connect (server, client);
+    success = s_can_connect (ctx, &server, &client);
     assert (success);
-    
+
     //  Test full client authentication using certificates
+    zcert_apply (server_cert, server);
+    zcert_apply (client_cert, client);
+    zsocket_set_curve_server (server, 1);
+    zsocket_set_curve_serverkey (client, server_key);
     zcert_save_public (client_cert, TESTDIR "/mycert.txt");
     zauth_configure_curve (auth, "*", TESTDIR);
-    success = s_can_connect (server, client);
+    success = s_can_connect (ctx, &server, &client);
     assert (success);
+    
     zcert_destroy (&server_cert);
     zcert_destroy (&client_cert);
 #   endif
 
     //  Remove the authenticator and check a normal connection works
     zauth_destroy (&auth);
-    success = s_can_connect (server, client);
+    success = s_can_connect (ctx, &server, &client);
     assert (success);
 
     zctx_destroy (&ctx);
