@@ -1,8 +1,8 @@
 /*  =========================================================================
-    zbeacon - LAN service announcement and discovery
+    zbeacon - LAN discovery and presence
 
     -------------------------------------------------------------------------
-    Copyright (c) 1991-2013 iMatix Corporation <www.imatix.com>
+    Copyright (c) 1991-2014 iMatix Corporation <www.imatix.com>
     Copyright other contributors as noted in the AUTHORS file.
 
     This file is part of CZMQ, the high-level C binding for 0MQ:
@@ -40,53 +40,16 @@
 */
 
 #include "../include/czmq.h"
-#if !defined (__WINDOWS__)
-#   include "platform.h"
-#endif
-
-#if defined (HAVE_LINUX_WIRELESS_H)
-#   include <linux/wireless.h>
-#else
-#   if defined (HAVE_NET_IF_H)
-#       include <net/if.h>
-#   endif
-#   if defined (HAVE_NET_IF_MEDIA_H)
-#       include <net/if_media.h>
-#   endif
-#endif
-
-#if defined (__UTYPE_SUNSOLARIS) || defined (__UTYPE_SUNOS)
-#   include <ifaddrs.h>
-#   include <sys/sockio.h>
-#endif
-
-#if defined (__WINDOWS__)
-#   if (_WIN32_WINNT < 0x0501)
-#       undef _WIN32_WINNT
-#       define _WIN32_WINNT 0x0501
-#   endif
-#   include <ws2tcpip.h>            //  For getnameinfo ()
-#   include <iphlpapi.h>            //  For GetAdaptersAddresses ()
-#endif
-
-//  Windows uses 
-#if !defined (__WINDOWS__)
-typedef int SOCKET;
-#   define closesocket close
-#   define INVALID_SOCKET -1
-#   define SOCKET_ERROR -1
+#include "platform.h"
+#if defined (__UNIX__) && defined (HAVE_GETIFADDRS) && defined (HAVE_FREEIFADDRS)
+#include <net/if.h>
 #endif
 
 //  Constants
-#define BEACON_MAX      255         //  Max size of beacon data
 #define INTERVAL_DFLT  1000         //  Default interval = 1 second
 
-//  Internet socket address structure
-typedef struct sockaddr_in inaddr_t;
-
 //  Structure of our class
 struct _zbeacon_t {
-    zctx_t *ctx;                //  Private 0MQ context
     void *pipe;                 //  Pipe through to backend agent
     char *hostname;             //  Our own address as string
 };
@@ -101,22 +64,22 @@ static void
 //  Create a new beacon
 
 zbeacon_t *
-zbeacon_new (int port_nbr)
+zbeacon_new (zctx_t *ctx, int port_nbr)
 {
     zbeacon_t *self = (zbeacon_t *) zmalloc (sizeof (zbeacon_t));
+    assert (self);
 
-    //  For now, we use a context per beacon instance
-    self->ctx = zctx_new ();
-
-    //  Start beacon background agent
-    self->pipe = zthread_fork (self->ctx, s_agent_task, NULL);
-    
-    //  Configure agent with arguments
-    zstr_send (self->pipe, "%d", port_nbr);
-
-    //  Agent replies with our host name
-    self->hostname = zstr_recv (self->pipe);
-
+    //  Start background agent and wait for it to initialize
+    assert (ctx);
+    self->pipe = zthread_fork (ctx, s_agent_task, NULL);
+    if (self->pipe) {
+        zstr_sendf (self->pipe, "%d", port_nbr);
+        self->hostname = zstr_recv (self->pipe);
+    }
+    else {
+        free (self);
+        self = NULL;
+    }
     return self;
 }
 
@@ -131,8 +94,8 @@ zbeacon_destroy (zbeacon_t **self_p)
     if (*self_p) {
         zbeacon_t *self = *self_p;
         zstr_send (self->pipe, "TERMINATE");
-        free (zstr_recv (self->pipe));
-        zctx_destroy (&self->ctx);
+        char *reply = zstr_recv (self->pipe);
+        zstr_free (&reply);
         free (self->hostname);
         free (self);
         *self_p = NULL;
@@ -148,7 +111,7 @@ zbeacon_set_interval (zbeacon_t *self, int interval)
 {
     assert (self);
     zstr_sendm (self->pipe, "INTERVAL");
-    zstr_send (self->pipe, "%d", interval);
+    zstr_sendf (self->pipe, "%d", interval);
 }
 
 
@@ -171,7 +134,7 @@ zbeacon_publish (zbeacon_t *self, byte *transmit, size_t size)
 {
     assert (self);
     assert (transmit);
-    assert (size > 0 && size <= BEACON_MAX);
+    assert (size > 0 && size <= UDP_FRAME_MAX);
     zmsg_t *msg = zmsg_new ();
     zmsg_addstr (msg, "PUBLISH");
     zmsg_addmem (msg, transmit, size);
@@ -197,7 +160,7 @@ void
 zbeacon_subscribe (zbeacon_t *self, byte *filter, size_t size)
 {
     assert (self);
-    assert (size <= BEACON_MAX);
+    assert (size <= UDP_FRAME_MAX);
     zmsg_t *msg = zmsg_new ();
     zmsg_addstr (msg, "SUBSCRIBE");
     zmsg_addmem (msg, filter, size);
@@ -239,6 +202,10 @@ zbeacon_hostname (zbeacon_t *self)
 
 //  --------------------------------------------------------------------------
 //  Self test of this class
+//
+// If this test fails with an unable to bind error, try specifying the default
+// interface to be used for zbeacon with the ZSYS_INTERFACE environment
+// variable.
 
 void
 zbeacon_test (bool verbose)
@@ -255,12 +222,12 @@ zbeacon_test (bool verbose)
     
     //  Create beacon to broadcast our service
     byte announcement [2] = { (port_nbr >> 8) & 0xFF, port_nbr & 0xFF };
-    zbeacon_t *service_beacon = zbeacon_new (9999);
+    zbeacon_t *service_beacon = zbeacon_new (ctx, 9999);
     zbeacon_set_interval (service_beacon, 100);
     zbeacon_publish (service_beacon, announcement, 2);
 
     //  Create beacon to lookup service
-    zbeacon_t *client_beacon = zbeacon_new (9999);
+    zbeacon_t *client_beacon = zbeacon_new (ctx, 9999);
     zbeacon_subscribe (client_beacon, NULL, 0);
 
     //  Wait for at most 1/2 second if there's no broadcast networking
@@ -273,15 +240,15 @@ zbeacon_test (bool verbose)
                         +  zframe_data (content) [1];
         assert (received_port == port_nbr);
         zframe_destroy (&content);
-        free (ipaddress);
+        zbeacon_silence (service_beacon);
+        zstr_free (&ipaddress);
     }
     zbeacon_destroy (&client_beacon);
     zbeacon_destroy (&service_beacon);
-    zctx_destroy (&ctx);
     
-    zbeacon_t *node1 = zbeacon_new (5670);
-    zbeacon_t *node2 = zbeacon_new (5670);
-    zbeacon_t *node3 = zbeacon_new (5670);
+    zbeacon_t *node1 = zbeacon_new (ctx, 5670);
+    zbeacon_t *node2 = zbeacon_new (ctx, 5670);
+    zbeacon_t *node3 = zbeacon_new (ctx, 5670);
 
     assert (*zbeacon_hostname (node1));
     assert (*zbeacon_hostname (node2));
@@ -313,8 +280,8 @@ zbeacon_test (bool verbose)
             char *ipaddress, *beacon;
             zstr_recvx (zbeacon_socket (node1), &ipaddress, &beacon, NULL);
             assert (streq (beacon, "NODE/2"));
-            free (ipaddress);
-            free (beacon);
+            zstr_free (&ipaddress);
+            zstr_free (&beacon);
         }
     }
     zpoller_destroy (&poller);
@@ -331,6 +298,8 @@ zbeacon_test (bool verbose)
     zbeacon_destroy (&node1);
     zbeacon_destroy (&node2);
     zbeacon_destroy (&node3);
+    
+    zctx_destroy (&ctx);
     //  @end
     printf ("OK\n");
 }
@@ -362,16 +331,10 @@ static agent_t *
     s_agent_new (void *pipe, int port_nbr);
 static void
     s_agent_destroy (agent_t **self_p);
-static void
-    s_handle_io_error (char *reason);
 static void
     s_get_interface (agent_t *self);
-static bool
-    s_wireless_nic (const char* name);
 static void
     s_api_command (agent_t *self);
-static void
-    s_beacon_send (agent_t *self);
 static void
     s_beacon_recv (agent_t *self);
 
@@ -387,9 +350,11 @@ s_agent_task (void *args, zctx_t *ctx, void *pipe)
 
     //  Create agent instance
     agent_t *self = s_agent_new (pipe, atoi (port_str));
-    free (port_str);
+    zstr_free (&port_str);
+    if (!self)
+        return;                 //  Give up if we couldn't initialize
 
-    while (!zctx_interrupted) {
+    while (!self->terminated) {
         //  Poll on API pipe and on UDP socket
         zmq_pollitem_t pollitems [] = {
             { self->pipe, 0, ZMQ_POLLIN, 0 },
@@ -408,14 +373,13 @@ s_agent_task (void *args, zctx_t *ctx, void *pipe)
             s_api_command (self);
         if (pollitems [1].revents & ZMQ_POLLIN)
             s_beacon_recv (self);
-        
+
         if (self->transmit
         &&  zclock_time () >= self->ping_at) {
-            s_beacon_send (self);
+            //  Send beacon to any listening peers
+            zsys_udp_send (self->udpsock, self->transmit, &self->broadcast);
             self->ping_at = zclock_time () + self->interval;
         }
-        if (self->terminated)
-            break;
     }
     s_agent_destroy (&self);
 }
@@ -434,149 +398,134 @@ s_agent_new (void *pipe, int port_nbr)
     self->interval = INTERVAL_DFLT;
 
     //  Create our UDP socket
-    self->udpsock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
-    if (self->udpsock == INVALID_SOCKET)
-        s_handle_io_error ("socket");
+    self->udpsock = zsys_udp_new (false);
+    if (self->udpsock == INVALID_SOCKET) {
+        free (self);        //  No more file handles - forget it
+        return NULL;
+    }
 
-    //  Ask operating system to let us do broadcasts from socket
-    int on = 1;
-    if (setsockopt (self->udpsock, SOL_SOCKET, SO_BROADCAST,
-                   (char *) &on, sizeof (on)) == SOCKET_ERROR)
-        s_handle_io_error ("setsockopt (SO_BROADCAST)");
-
-    //  Allow multiple owners to bind to socket; incoming
-    //  messages will replicate to each owner
-    if (setsockopt (self->udpsock, SOL_SOCKET, SO_REUSEADDR,
-                   (char *) &on, sizeof (on)) == SOCKET_ERROR)
-        s_handle_io_error ("setsockopt (SO_REUSEADDR)");
-
-#if defined (SO_REUSEPORT)
-    //  On some platforms we have to ask to reuse the port
-    if (setsockopt (self->udpsock, SOL_SOCKET, SO_REUSEPORT,
-                    (char *) &on, sizeof (on)) == SOCKET_ERROR)
-        s_handle_io_error ("setsockopt (SO_REUSEPORT)");
-#endif
     //  PROBLEM: this design will not survive the network interface
     //  being killed and restarted while the program is running
-
-    //  Bind to the port on all interfaces
-    inaddr_t sockaddr = { 0 };
-    sockaddr.sin_family = AF_INET;
-    sockaddr.sin_port = htons (self->port_nbr);
-    sockaddr.sin_addr.s_addr = htonl (INADDR_ANY);
-    if (bind (self->udpsock, (struct sockaddr *) &sockaddr, sizeof (sockaddr)) == SOCKET_ERROR)
-        s_handle_io_error ("bind");
-
     //  Get the network interface
     s_get_interface (self);
 
+    //  Bind to the port on all interfaces
+#if (defined (__WINDOWS__))
+    inaddr_t sockaddr = self->address;
+#else
+    inaddr_t sockaddr = self->broadcast;
+#endif
+    if (bind (self->udpsock, (struct sockaddr *) &sockaddr, sizeof (sockaddr)) == SOCKET_ERROR)
+        zsys_socket_error ("bind");
+
     //  Send our hostname back to API
-    char hostname [INET_ADDRSTRLEN];
-    getnameinfo ((struct sockaddr *) &self->address, sizeof (self->address),
-                 hostname, INET_ADDRSTRLEN, NULL, 0, NI_NUMERICHOST);
+    //  PROBLEM: this hostname will not be accurate when the node
+    //  has more than one active interface.
+    char hostname [255];
+    int rc = getnameinfo ((struct sockaddr *) &self->address, sizeof (self->address),
+                           hostname, 255, NULL, 0, NI_NUMERICHOST);
+    if (rc) {
+        puts (gai_strerror (rc));
+        strcpy (hostname, "127.0.0.1");
+    }
     zstr_send (pipe, hostname);
     
     return self;
 }
 
 
-//  Handle error from I/O operation
-
-static void
-s_handle_io_error (char *reason)
-{
-#if defined (__WINDOWS__)
-    switch (WSAGetLastError ()) {
-        case WSAEINTR:        errno = EINTR;      break;
-        case WSAEBADF:        errno = EBADF;      break;
-        case WSAEWOULDBLOCK:  errno = EAGAIN;     break;
-        case WSAEINPROGRESS:  errno = EAGAIN;     break;
-        case WSAENETDOWN:     errno = ENETDOWN;   break;
-        case WSAECONNRESET:   errno = ECONNRESET; break;
-        case WSAECONNABORTED: errno = EPIPE;      break;
-        case WSAESHUTDOWN:    errno = ECONNRESET; break;
-        case WSAEINVAL:       errno = EPIPE;      break;
-        default:              errno = GetLastError ();
-    }
-#endif
-    if (errno == EAGAIN
-    ||  errno == ENETDOWN
-    ||  errno == EHOSTUNREACH
-    ||  errno == ENETUNREACH
-    ||  errno == EINTR
-    ||  errno == EPIPE
-    ||  errno == ECONNRESET
-#if !defined (__WINDOWS__)
-    ||  errno == EPROTO
-    ||  errno == ENOPROTOOPT
-    ||  errno == EHOSTDOWN
-    ||  errno == EOPNOTSUPP
-    ||  errno == EWOULDBLOCK
-#endif
-#if defined (ENONET)
-    ||  errno == ENONET
-#endif
-    )
-        return;             //  Ignore error and try again
-    else {
-        zclock_log ("E: (UDP) error '%s' on %s", strerror (errno), reason);
-        assert (false);
-    }
-}
-
-
-//  Get the actual network interface we're working on
+//  Get the actual network interface we're working on.
 //  Currently implemented for POSIX and for Windows
 
 static void
 s_get_interface (agent_t *self)
 {
-#if defined (__UNIX__)
-#   if defined (HAVE_GETIFADDRS) && defined (HAVE_FREEIFADDRS)
+#if defined (__UNIX__) && defined (HAVE_GETIFADDRS) && defined (HAVE_FREEIFADDRS)
     struct ifaddrs *interfaces;
     if (getifaddrs (&interfaces) == 0) {
+        int num_interfaces = 0;
         struct ifaddrs *interface = interfaces;
         while (interface) {
-            //  Hopefully the last interface will be WiFi or Ethernet
-            if (interface->ifa_addr && 
-                interface->ifa_broadaddr && // on Solaris, loopback interfaces have a NULL in ifa_broadaddr
-                (interface->ifa_addr->sa_family == AF_INET)) {
+            //  On Solaris, loopback interfaces have a NULL in ifa_broadaddr
+            if  (interface->ifa_broadaddr
+            && !(interface->ifa_flags & IFF_LOOPBACK)       //  Ignore loopback interface
+            &&  (interface->ifa_flags & IFF_BROADCAST)      //  Only use interfaces that have BROADCAST
+            && !(interface->ifa_flags & IFF_POINTOPOINT)    //  Ignore point to point interfaces.
+            &&   interface->ifa_addr
+            &&  (interface->ifa_addr->sa_family == AF_INET)) {
                 self->address = *(inaddr_t *) interface->ifa_addr;
                 self->broadcast = *(inaddr_t *) interface->ifa_broadaddr;
                 self->broadcast.sin_port = htons (self->port_nbr);
-                if (streq (interface->ifa_name, zsys_interface ())
-                ||  s_wireless_nic (interface->ifa_name))
+
+                //  If the returned broadcast address is the same as source
+                //  address, build the broadcast address from the source
+                //  address and netmask.
+                if (self->address.sin_addr.s_addr == self->broadcast.sin_addr.s_addr)
+                    self->broadcast.sin_addr.s_addr
+                        |= ~(((inaddr_t *) interface->ifa_netmask)->sin_addr.s_addr);
+
+                //  Keep track of the number of interfaces on this host
+                if (self->address.sin_addr.s_addr != ntohl (INADDR_LOOPBACK))
+                    num_interfaces++;
+
+                //  If this is the specified interface then move on
+                if (streq (interface->ifa_name, zsys_interface ()))
                     break;
             }
             interface = interface->ifa_next;
         }
+        if (strlen (zsys_interface ()) == 0) {
+            //  Subnet broadcast addresses don't work on some platforms but 
+            //  is assumed to work if the interface is specified.
+            self->broadcast.sin_family = AF_INET;
+            self->broadcast.sin_addr.s_addr = INADDR_BROADCAST;
+            self->broadcast.sin_port = htons (self->port_nbr);
+            if (num_interfaces > 1) {
+                //  Our source address is unknown in this case so set it to
+                //  INADDR_ANY so self->hostname isn't set to an incorrect IP.
+                self->address = self->broadcast;
+                self->address.sin_addr.s_addr = INADDR_ANY;
+            }
+        }
     }
     freeifaddrs (interfaces);
-#   else
+
+#else
+    //  Our default if we fail to find an interface
+    self->broadcast.sin_family = AF_INET;
+    self->broadcast.sin_addr.s_addr = INADDR_BROADCAST;
+    self->broadcast.sin_port = htons (self->port_nbr);
+
+    //  Our source address is unknown in this case so set it to
+    //  INADDR_ANY so self->hostname isn't set to an incorrect IP.
+    self->address = self->broadcast;
+    self->address.sin_addr.s_addr = INADDR_ANY;
+    if (strlen (zsys_interface ()) == 0)
+        return;         //  Use defaults
+
+#   if defined (__UNIX__)
     struct ifreq ifr;
     memset (&ifr, 0, sizeof (ifr));
 
     int sock = 0;
-    if ((sock = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
-        s_handle_io_error ("socket");
+    if ((sock = socket (AF_INET, SOCK_DGRAM, 0)) == -1)
+        return;         //  Use defaults
 
     //  Get interface address
     ifr.ifr_addr.sa_family = AF_INET;
     strncpy (ifr.ifr_name, zsys_interface (), sizeof (ifr.ifr_name));
     int rc = ioctl (sock, SIOCGIFADDR, (caddr_t) &ifr, sizeof (struct ifreq));
     if (rc == -1)
-        s_handle_io_error ("siocgifaddr");
+        zsys_socket_error ("siocgifaddr");
 
     //  Get interface broadcast address
-    memcpy (&self->address, ((inaddr_t *) &ifr.ifr_addr),
-        sizeof (inaddr_t));
+    memcpy (&self->address, ((inaddr_t *) &ifr.ifr_addr), sizeof (inaddr_t));
     rc = ioctl (sock, SIOCGIFBRDADDR, (caddr_t) &ifr, sizeof (struct ifreq));
     if (rc == -1)
-        s_handle_io_error ("siocgifbrdaddr");
+        zsys_socket_error ("siocgifbrdaddr");
 
     memcpy (&self->broadcast, ((inaddr_t *) &ifr.ifr_broadaddr), sizeof (inaddr_t));
     close (sock);
-#   endif
 
 #   elif defined (__WINDOWS__)
     ULONG addr_size = 0;
@@ -597,7 +546,8 @@ s_get_interface (agent_t *self)
         if (pUnicast && pPrefix) {
             self->address = *(inaddr_t *)(pUnicast->Address.lpSockaddr);
             self->broadcast = *(inaddr_t *)(pPrefix->Address.lpSockaddr);
-            self->broadcast.sin_addr.s_addr |= htonl ((1 << (32 - pPrefix->PrefixLength)) - 1);
+            self->broadcast.sin_addr.s_addr
+                |= htonl ((1 << (32 - pPrefix->PrefixLength)) - 1);
         }
         cur_address = cur_address->Next;
     }
@@ -605,40 +555,7 @@ s_get_interface (agent_t *self)
 #   else
 #       error "Interface detection TBD on this operating system"
 #   endif
-
-    //  Set broadcast address and port
-    self->broadcast.sin_addr.s_addr = INADDR_BROADCAST;
-    self->broadcast.sin_port = htons (self->port_nbr);
-}
-
-//  Check if a given NIC name is wireless
-
-static bool
-s_wireless_nic (const char *name)
-{
-    SOCKET udpsock = socket (AF_INET, SOCK_DGRAM, 0);
-    if (udpsock == INVALID_SOCKET)
-        s_handle_io_error ("socket");
-
-    bool is_nic = false;
-#if defined (SIOCGIFMEDIA)
-    struct ifmediareq ifmr;
-    memset (&ifmr, 0, sizeof (struct ifmediareq));
-    strncpy(ifmr.ifm_name, name, sizeof (ifmr.ifm_name));
-    int res = ioctl (udpsock, SIOCGIFMEDIA, (caddr_t) &ifmr);
-    if (res != -1)
-        is_nic = (IFM_TYPE (ifmr.ifm_current) == IFM_IEEE80211);
-
-#elif defined (SIOCGIWNAME)
-    struct iwreq wrq;
-    memset (&wrq, 0, sizeof (struct iwreq));
-    strncpy (wrq.ifr_name, name, sizeof (wrq.ifr_name));
-    int res = ioctl (udpsock, SIOCGIWNAME, (caddr_t) &wrq);
-    if (res != -1)
-        is_nic = true;
 #endif
-    closesocket (udpsock);
-    return is_nic;
 }
 
 //  Handle command from API
@@ -650,7 +567,7 @@ s_api_command (agent_t *self)
     if (streq (command, "INTERVAL")) {
         char *interval = zstr_recv (self->pipe);
         self->interval = atoi (interval);
-        free (interval);
+        zstr_free (&interval);
     }
     else
     if (streq (command, "NOECHO"))
@@ -682,9 +599,10 @@ s_api_command (agent_t *self)
     else
         printf ("E: unexpected API command '%s'\n", command);
     
-    free (command);
+    zstr_free (&command);
 }
 
+
 //  Receive and filter the waiting beacon
 
 static void
@@ -692,75 +610,37 @@ s_beacon_recv (agent_t *self)
 {
     assert (self);
 
-    socklen_t si_len = sizeof (inaddr_t);
-    inaddr_t sender;
-    byte buffer [BEACON_MAX];
-    ssize_t size = recvfrom (
-        self->udpsock,
-        (char *) buffer, BEACON_MAX,
-        0,      //  Flags
-        (struct sockaddr *) &sender, &si_len);
-    if (size == SOCKET_ERROR)
-        s_handle_io_error ("recvfrom");
-    
-    //  Get sender address as printable string
     char peername [INET_ADDRSTRLEN];
-#if (defined (__WINDOWS__))
-    getnameinfo ((struct sockaddr *) &sender, si_len,
-                peername, INET_ADDRSTRLEN, NULL, 0, NI_NUMERICHOST);
-#else
-    inet_ntop (AF_INET, &sender.sin_addr, peername, si_len);
-#endif
+    zframe_t *frame = zsys_udp_recv (self->udpsock, peername);
     
     //  If filter is set, check that beacon matches it
     bool is_valid = false;
     if (self->filter) {
         byte  *filter_data = zframe_data (self->filter);
         size_t filter_size = zframe_size (self->filter);
-        if ((size_t) size >= filter_size
-        &&  memcmp (buffer, filter_data, filter_size) == 0)
+        if (zframe_size (frame) >= filter_size
+        && memcmp (zframe_data (frame), filter_data, filter_size) == 0)
             is_valid = true;
     }
     //  If valid, check for echoed beacons (i.e. our own broadcast)
     if (is_valid && self->noecho && self->transmit) {
         byte  *transmit_data = zframe_data (self->transmit);
         size_t transmit_size = zframe_size (self->transmit);
-        if (size == transmit_size && memcmp (buffer, transmit_data, transmit_size) == 0)
+        if (zframe_size (frame) == transmit_size
+        && memcmp (zframe_data (frame), transmit_data, transmit_size) == 0)
             is_valid = false;
     }
     //  If still a valid beacon, send on to the API
     if (is_valid) {
         zmsg_t *msg = zmsg_new ();
         zmsg_addstr (msg, peername);
-        zmsg_addmem (msg, buffer, size);
+        zmsg_add (msg, frame);
         zmsg_send (&msg, self->pipe);
     }
+    else
+        zframe_destroy (&frame);
 }
 
-//  Send beacon to any listening peers
-
-static void
-s_beacon_send (agent_t *self)
-{
-    //  Send UDP broadcast packet now
-    assert (self->transmit);
-    sendto (
-        self->udpsock,
-        (char *) zframe_data (self->transmit), zframe_size (self->transmit),
-        0,      //  Flags
-        (struct sockaddr *) &self->broadcast, sizeof (inaddr_t));
-
-    //  Sending can fail if the OS is blocking multicast. In such cases we
-    //  don't try to report the error. We might log this or send to an error
-    //  console at some point.
-    //  ssize_t size = sendto (
-    //      self->udpsock,
-    //      (char *) zframe_data (self->transmit), zframe_size (self->transmit),
-    //      0,      //  Flags
-    //      (struct sockaddr *) &self->broadcast, sizeof (inaddr_t));
-    //  if (size == SOCKET_ERROR)
-    //      s_handle_io_error ("sendto");
-}
 
 //  Destroy agent instance
 
