@@ -2,7 +2,7 @@
     zcertstore - work with CURVE security certificate stores
 
     -------------------------------------------------------------------------
-    Copyright (c) 1991-2013 iMatix Corporation <www.imatix.com>
+    Copyright (c) 1991-2014 iMatix Corporation <www.imatix.com>
     Copyright other contributors as noted in the AUTHORS file.
 
     This file is part of CZMQ, the high-level C binding for 0MQ:
@@ -71,13 +71,12 @@ struct _zcertstore_t {
 //  absent, and created later, or modified at any time. The certificate store 
 //  is automatically refreshed on any zcertstore_lookup() call. If the 
 //  location is specified as NULL, creates a pure-memory store, which you 
-//  can work with by inserting certificates at runtime. The location is
-//  treated as a printf format.
+//  can work with by inserting certificates at runtime.
 
 static void s_load_certs_from_disk (zcertstore_t *self);
 
 zcertstore_t *
-zcertstore_new (char *location, ...)
+zcertstore_new (const char *location)
 {
     zcertstore_t *self = (zcertstore_t *) zmalloc (sizeof (zcertstore_t));
     assert (self);
@@ -85,10 +84,7 @@ zcertstore_new (char *location, ...)
     self->cert_list = zlist_new ();
     self->cert_hash = zhash_new ();
     if (location) {
-        va_list argptr;
-        va_start (argptr, location);
-        self->location = zsys_vprintf (location, argptr);
-        va_end (argptr);
+        self->location = strdup (location);
         s_load_certs_from_disk (self);
     }
     return self;
@@ -133,7 +129,7 @@ s_load_certs_from_disk (zcertstore_t *self)
                     zcertstore_insert (self, &cert);
             }
         }
-        free (filelist);
+        zdir_flatten_free (&filelist);
         self->modified = zdir_modified (dir);
         self->count = zdir_count (dir);
         self->cursize = zdir_cursize (dir);
@@ -170,7 +166,7 @@ zcertstore_destroy (zcertstore_t **self_p)
 //  else returns NULL. The public key is provided in Z85 text format.
 
 zcert_t *
-zcertstore_lookup (zcertstore_t *self, char *public_key)
+zcertstore_lookup (zcertstore_t *self, const char *public_key)
 {
     //  If directory has changed, reload all certificates
     if (self->location) {
@@ -202,25 +198,34 @@ zcertstore_insert (zcertstore_t *self, zcert_t **cert_p)
 
 
 //  --------------------------------------------------------------------------
-//  Print out list of certificates in store to stdout, for debugging
-//  purposes.
+//  Print list of certificates in store to open stream
 
 void
-zcertstore_dump (zcertstore_t *self)
+zcertstore_fprint (zcertstore_t *self, FILE *file)
 {
     if (self->location)
-        printf ("Certificate store at %s:\n", self->location);
+        fprintf (file, "Certificate store at %s:\n", self->location);
     else
-        printf ("Certificate store\n");
-        
+        fprintf (file, "Certificate store\n");
+
     zcert_t *cert = (zcert_t *) zlist_first (self->cert_list);
     while (cert) {
-        zcert_dump (cert);
+        zcert_fprint (cert, file);
         cert = (zcert_t *) (zcert_t *) zlist_next (self->cert_list);
     }
 }
 
 
+//  --------------------------------------------------------------------------
+//  Print list of certificates in store to stdout
+
+void
+zcertstore_print (zcertstore_t *self)
+{
+    zcertstore_fprint (self, stdout);
+}
+
+
 //  --------------------------------------------------------------------------
 //  Selftest
 
@@ -235,8 +240,9 @@ zcertstore_test (bool verbose)
     zsys_dir_create (TESTDIR);
     
     //  Load certificate store from disk; it will be empty
-    zcertstore_t *certstore = zcertstore_new ("%s", TESTDIR);
+    zcertstore_t *certstore = zcertstore_new (TESTDIR);
     
+#   if defined (HAVE_LIBSODIUM)
     //  Create a single new certificate and save to disk
     zcert_t *cert = zcert_new ();
     char *client_key = strdup (zcert_public_txt (cert));
@@ -249,11 +255,11 @@ zcertstore_test (bool verbose)
     assert (cert);
     assert (streq (zcert_meta (cert, "name"), "John Doe"));
     free (client_key);
+#   endif
     if (verbose)
-        zcertstore_dump (certstore);
-        
+        zcertstore_print (certstore);
     zcertstore_destroy (&certstore);
-    
+
     //  Delete all test files
     zdir_t *dir = zdir_new (TESTDIR, NULL);
     zdir_remove (dir, true);
