@@ -2,7 +2,7 @@
     zconfig - work with config files written in rfc.zeromq.org/spec:4/ZPL.
 
     -------------------------------------------------------------------------
-    Copyright (c) 1991-2013 iMatix Corporation <www.imatix.com>
+    Copyright (c) 1991-2014 iMatix Corporation <www.imatix.com>
     Copyright other contributors as noted in the AUTHORS file.
 
     This file is part of CZMQ, the high-level C binding for 0MQ:
@@ -29,6 +29,39 @@
     This implements rfc.zeromq.org/spec:4/ZPL, which is a simple structured
     text format for configuration files.
 @discuss
+    Here is an example ZPL stream and corresponding config structure:
+
+    context
+        iothreads = 1
+        verbose = 1      #   Ask for a trace
+    main
+        type = zqueue    #  ZMQ_DEVICE type
+        frontend
+            option
+                hwm = 1000
+                swap = 25000000     #  25MB
+            bind = 'inproc://addr1'
+            bind = 'ipc://addr2'
+        backend
+            bind = inproc://addr3
+
+    root                    Down = child
+      |                     Across = next
+      v
+    context-->main
+      |         |
+      |         v
+      |       type=queue-->frontend-->backend
+      |                      |          |
+      |                      |          v
+      |                      |        bind=inproc://addr3
+      |                      v
+      |                    option-->bind=inproc://addr1-->bind=ipc://addr2
+      |                      |
+      |                      v
+      |                    hwm=1000-->swap=25000000
+      v
+    iothreads=1-->verbose=false
 @end
 */
 
@@ -58,6 +91,8 @@ static char *
     s_collect_value (char **start, int lineno);
 static int
     s_config_save (zconfig_t *self, void *arg, int level);
+static int
+    s_config_save_chunk (zconfig_t *self, void *arg, int level);
 
 
 //  --------------------------------------------------------------------------
@@ -66,7 +101,7 @@ static int
 //  Optionally attach new config to parent config, as first or next child.
 
 zconfig_t *
-zconfig_new (char *name, zconfig_t *parent)
+zconfig_new (const char *name, zconfig_t *parent)
 {
     zconfig_t *self = (zconfig_t *) zmalloc (sizeof (zconfig_t));
     zconfig_set_name (self, name);
@@ -138,13 +173,13 @@ zconfig_value (zconfig_t *self)
 //  and ignored.
 
 void
-zconfig_put (zconfig_t *self, char *path, char *value)
+zconfig_put (zconfig_t *self, const char *path, const char *value)
 {
     if (*path == '/')
         path++;
         
     //  Check length of next path segment
-    char *slash = strchr (path, '/');
+    const char *slash = strchr (path, '/');
     int length = strlen (path);
     if (slash)
         length = slash - path;
@@ -177,7 +212,7 @@ zconfig_put (zconfig_t *self, char *path, char *value)
 //  Set new name for config item; this may be null.
 
 void
-zconfig_set_name (zconfig_t *self, char *name)
+zconfig_set_name (zconfig_t *self, const char *name)
 {
     assert (self);
     free (self->name);
@@ -192,7 +227,7 @@ zconfig_set_name (zconfig_t *self, char *name)
 //  by the string.
 
 void
-zconfig_set_value (zconfig_t *self, char *format, ...)
+zconfig_set_value (zconfig_t *self, const char *format, ...)
 {
     assert (self);
     free (self->value);
@@ -233,12 +268,12 @@ zconfig_next (zconfig_t *self)
 //  Find a config item along a path; leading slash is optional and ignored.
 
 zconfig_t *
-zconfig_locate (zconfig_t *self, char *path)
+zconfig_locate (zconfig_t *self, const char *path)
 {
     //  Check length of next path segment
     if (*path == '/')
         path++;
-    char *slash = strchr (path, '/');
+    const char *slash = strchr (path, '/');
     int length = strlen (path);
     if (slash)
         length = slash - path;
@@ -263,7 +298,7 @@ zconfig_locate (zconfig_t *self, char *path)
 //  Resolve a config path into a string value
 
 char *
-zconfig_resolve (zconfig_t *self, char *path, char *default_value)
+zconfig_resolve (zconfig_t *self, const char *path, const char *default_value)
 {
     zconfig_t *item = zconfig_locate (self, path);
     if (item)
@@ -324,66 +359,114 @@ zconfig_execute (zconfig_t *self, zconfig_fct handler, void *arg)
 
 
 //  --------------------------------------------------------------------------
-//  Dump the config file to stderr for tracing
+//  Load a config tree from a specified ZPL text file
 
-void
-zconfig_dump (zconfig_t *self)
+zconfig_t *
+zconfig_load (const char *filename)
 {
-    zconfig_execute (self, s_config_save, stderr);
+    //  Load entire file into memory as a chunk, then process it
+    zconfig_t *self = NULL;
+    zfile_t *file = zfile_new (NULL, filename);
+    if (zfile_input (file) == 0) {
+        zchunk_t *chunk = zfile_read (file, zfile_cursize (file), 0);
+        if (chunk)
+            self = zconfig_chunk_load (chunk);
+        zchunk_destroy (&chunk);
+    }
+    zfile_destroy (&file);
+    return self;
 }
 
 
 //  --------------------------------------------------------------------------
-//  Load a config item tree from a specified ZPL text file
-//
-//  Here is an example ZPL stream and corresponding config structure:
-//
-//  context
-//      iothreads = 1
-//      verbose = 1      #   Ask for a trace
-//  main
-//      type = zqueue    #  ZMQ_DEVICE type
-//      frontend
-//          option
-//              hwm = 1000
-//              swap = 25000000     #  25MB
-//          bind = 'inproc://addr1'
-//          bind = 'ipc://addr2'
-//      backend
-//          bind = inproc://addr3
-//
-//  root                    Down = child
-//    |                     Across = next
-//    v
-//  context-->main
-//    |         |
-//    |         v
-//    |       type=queue-->frontend-->backend
-//    |                      |          |
-//    |                      |          v
-//    |                      |        bind=inproc://addr3
-//    |                      v
-//    |                    option-->bind=inproc://addr1-->bind=ipc://addr2
-//    |                      |
-//    |                      v
-//    |                    hwm=1000-->swap=25000000
-//    v
-//  iothreads=1-->verbose=false
+//  Save a config tree to a specified ZPL text file, where a filename
+//  "-" means dump to standard output.
+
+int
+zconfig_save (zconfig_t *self, const char *filename)
+{
+    assert (self);
+
+    int rc = 0;
+    if (streq (filename, "-")) {
+        //  "-" means write to stdout
+        rc = zconfig_execute (self, s_config_save, stdout);
+    }
+    else {
+        FILE *file;
+        file = fopen (filename, "w");
+        if (file)
+            rc = zconfig_execute (self, s_config_save, file);
+        else
+            rc = -1;          //  File not writeable
+        fflush (file);
+        fclose (file);
+    }
+    return rc;
+}
+
+//  Save an item
+
+static int
+s_config_save (zconfig_t *self, void *arg, int level)
+{
+    assert (self);
+    assert (arg);
+
+    FILE *file = (FILE *) arg;
+    //  Save any comments on the item
+    if (self->comments) {
+        char *comment = (char *) zlist_first (self->comments);
+        while (comment) {
+            fprintf (file, "#%s\n", comment);
+            comment = (char *) zlist_next (self->comments);
+        }
+        //  Blank line after comments is nice
+        fprintf (file, "\n");
+    }
+    if (level > 0) {
+        if (self->value)
+            fprintf (file, "%*s%s = \"%s\"\n", (level - 1) * 4, "",
+                self->name? self->name: "(Unnamed)", self->value);
+        else
+            fprintf (file, "%*s%s\n", (level - 1) * 4, "",
+                self->name? self->name: "(Unnamed)");
+    }
+    return 0;
+}
+
+
+//  --------------------------------------------------------------------------
+//  Load a config tree from a memory chunk
 
 zconfig_t *
-zconfig_load (char *filename)
+zconfig_chunk_load (zchunk_t *chunk)
 {
-    FILE *file = fopen (filename, "r");
-    if (!file)
-        return NULL;            //  File not found, or unreadable
-
-    //  Parse the file line by line
+    //  Parse the chunk line by line
     zconfig_t *self = zconfig_new ("root", NULL);
-    char cur_line [1024];
     bool valid = true;
     int lineno = 0;
+    char *data_ptr = (char *) zchunk_data (chunk);
+    size_t remaining = zchunk_size (chunk);
     
-    while (fgets (cur_line, 1024, file)) {
+    while (remaining) {
+        //  Copy stuff into cur_line; not fastest but safest option
+        //  since chunk may not be null terminated, etc.
+        char *eoln = (char *) memchr (data_ptr, '\n', remaining);
+        size_t cur_size;
+        if (eoln)
+            cur_size = eoln - data_ptr;
+        else
+            cur_size = remaining;
+        
+        if (cur_size > 1024)
+            cur_size = 1024;
+        char cur_line [1024 + 1];
+        memcpy (cur_line, data_ptr, cur_size);
+        cur_line [cur_size] = '\0';
+        data_ptr = eoln? eoln + 1: NULL;
+        remaining -= cur_size + 1;
+        
         //  Trim line
         int length = strlen (cur_line);
         while (length && isspace ((byte) cur_line [length - 1]))
@@ -423,7 +506,7 @@ zconfig_load (char *filename)
                     item->value = value;
                 }
                 else {
-                    fprintf (stderr, "E: (%d) indentation error\n", lineno);
+                    fprintf (stderr, "zconfig E: (%d) indentation error\n", lineno);
                     free (value);
                     valid = false;
                 }
@@ -432,15 +515,14 @@ zconfig_load (char *filename)
         else
         if (s_verify_eoln (scanner, lineno))
             valid = false;
-            
+
         free (name);
         if (!valid)
             break;
     }
-    //  Either the whole ZPL file is valid or none of it is
+    //  Either the whole ZPL stream is valid or none of it is
     if (!valid)
         zconfig_destroy (&self);
-    fclose (file);
     return self;
 }
 
@@ -455,7 +537,7 @@ s_collect_level (char **start, int lineno)
         readptr++;
     int level = (readptr - *start) / 4;
     if (level * 4 != readptr - *start) {
-        fprintf (stderr, "E: (%d) indent 4 spaces at once\n", lineno);
+        fprintf (stderr, "zconfig E: (%d) indent 4 spaces at once\n", lineno);
         level = -1;
     }
     *start = readptr;
@@ -465,9 +547,9 @@ s_collect_level (char **start, int lineno)
 //  Collect property name
 
 static bool
-s_is_namechar (char thischar) 
+s_is_namechar (char thischar)
 {
-    return (isalnum (thischar) 
+    return (isalnum (thischar)
          || thischar == '$'
          || thischar == '-'
          || thischar == '_'
@@ -489,10 +571,10 @@ s_collect_name (char **start, int lineno)
     char *name = (char *) zmalloc (length + 1);
     memcpy (name, readptr, length);
     name [length] = 0;
-    
+
     if (length > 0
     && (name [0] == '/' || name [length - 1] == '/')) {
-        fprintf (stderr, "E: (%d) '/' not valid at name start or end\n", lineno);
+        fprintf (stderr, "zconfig E: (%d) '/' not valid at name start or end\n", lineno);
         free (name);
     }
     return name;
@@ -547,7 +629,7 @@ s_collect_value (char **start, int lineno)
                 rc = s_verify_eoln (endquote + 1, lineno);
             }
             else {
-                fprintf (stderr, "E: (%d) missing %c\n", lineno, *readptr);
+                fprintf (stderr, "zconfig E: (%d) missing %c\n", lineno, *readptr);
                 rc = -1;
             }
         }
@@ -573,13 +655,75 @@ s_collect_value (char **start, int lineno)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Save a config tree to a new memory chunk
+
+zchunk_t *
+zconfig_chunk_save (zconfig_t *self)
+{
+    assert (self);
+    size_t out_count = (size_t) zconfig_execute (self, s_config_save_chunk, NULL);
+    printf ("OUT COUNT: %zd\n", out_count);
+    zchunk_t *chunk = zchunk_new (NULL, out_count);
+    zconfig_execute (self, s_config_save_chunk, zchunk_data (chunk));
+    return chunk;
+}
+
+
+//  Save an item to memory; stores the item and value with indent
+//  and newline, at the indicated location, if not null. Always
+//  returns the number of bytes (that would be) written.
+
+static int
+s_config_save_chunk (zconfig_t *self, void *arg, int level)
+{
+    assert (self);
+    assert (arg);
+
+    int out_count = 0;
+    zchunk_t *chunk = (zchunk_t *) arg;
+
+    //  Store any comments on the item
+    if (self->comments) {
+        char *comment = (char *) zlist_first (self->comments);
+        while (comment) {
+            char curline [1024];
+            snprintf (curline, 1024, "#%s\n", comment);
+            out_count += strlen (curline);
+            if (chunk)
+                zchunk_append (chunk, curline, strlen (curline));
+            comment = (char *) zlist_next (self->comments);
+        }
+        //  Blank line after comments is nice
+        out_count++;
+        if (chunk)
+            zchunk_append (chunk, "\n", 1);
+    }
+    if (level > 0) {
+        char curline [1024];
+        if (self->value)
+            snprintf (curline, 1024, "%*s%s = \"%s\"\n",
+                     (level - 1) * 4, "",
+                      self->name? self->name: "(Unnamed)", self->value);
+        else
+            snprintf (curline, 1024, "%*s%s\n",
+                     (level - 1) * 4, "",
+                      self->name? self->name: "(Unnamed)");
+        out_count += strlen (curline);
+        if (chunk)
+            zchunk_append (chunk, curline, strlen (curline));
+    }
+    return out_count;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Add comment to config item before saving to disk. You can add as many
 //  comment lines as you like. If you use a null format, all comments are 
 //  deleted.
 
 void
-zconfig_comment (zconfig_t *self, char *format, ...)
+zconfig_set_comment (zconfig_t *self, const char *format, ...)
 {
     if (format) {
         if (!self->comments) {
@@ -600,60 +744,33 @@ zconfig_comment (zconfig_t *self, char *format, ...)
 
 
 //  --------------------------------------------------------------------------
-//  Save a config item tree to a specified ZPL text file, where a filename
-//  "-" means dump to standard output.
+//  Return comments of config item, as zlist.
 
-int
-zconfig_save (zconfig_t *self, char *filename)
+zlist_t *
+zconfig_comments (zconfig_t *self)
 {
     assert (self);
-
-    int rc = 0;
-    if (streq (filename, "-")) {
-        //  "-" means write to stdout
-        rc = zconfig_execute (self, s_config_save, stdout);
-    }
-    else {
-        FILE *file;
-        file = fopen (filename, "w");
-        if (file)
-            rc = zconfig_execute (self, s_config_save, file);
-        else
-            rc = -1;          //  File not writeable
-        fflush (file);
-        fclose (file);
-    }
-    return rc;
+    return self->comments;
 }
 
-//  Save an item
 
-static int
-s_config_save (zconfig_t *self, void *arg, int level)
+//  --------------------------------------------------------------------------
+//  Print the config file to open stream
+
+void
+zconfig_fprint (zconfig_t *self, FILE *file)
 {
-    assert (self);
-    assert (arg);
+    zconfig_execute (self, s_config_save, file);
+}
 
-    FILE *file = (FILE *) arg;
-    //  Save any comments on the item
-    if (self->comments) {
-        char *comment = (char *) zlist_first (self->comments);
-        while (comment) {
-            fprintf (file, "#%s\n", comment);
-            comment = (char *) zlist_next (self->comments);
-        }
-        //  Blank line after comments is nice
-        fprintf (file, "\n");
-    }
-    if (level > 0) {
-        if (self->value)
-            fprintf (file, "%*s%s = \"%s\"\n", (level - 1) * 4, "",
-                self->name? self->name: "(Unnamed)", self->value);
-        else
-            fprintf (file, "%*s%s\n", (level - 1) * 4, "",
-                self->name? self->name: "(Unnamed)");
-    }
-    return 0;
+
+//  --------------------------------------------------------------------------
+//  Print the config file to stdout
+
+void
+zconfig_print (zconfig_t *self)
+{
+    zconfig_fprint (self, stdout);
 }
 
 
@@ -679,8 +796,9 @@ zconfig_test (bool verbose)
     item = zconfig_new ("name", section);
     zconfig_set_value (item, "Justin Kayce");
     zconfig_put (root, "/curve/secret-key", "Top Secret");
-    zconfig_comment (root, "   CURVE certificate");
-    zconfig_comment (root, "   -----------------");
+    zconfig_set_comment (root, "   CURVE certificate");
+    zconfig_set_comment (root, "   -----------------");
+    assert (zconfig_comments (root));
     zconfig_save (root, TESTDIR "/test.cfg");
     zconfig_destroy (&root);
     root = zconfig_load (TESTDIR "/test.cfg");
