@@ -2,7 +2,7 @@
     zhash - generic type-free hash container
 
     -------------------------------------------------------------------------
-    Copyright (c) 1991-2013 iMatix Corporation <www.imatix.com>
+    Copyright (c) 1991-2014 iMatix Corporation <www.imatix.com>
     Copyright other contributors as noted in the AUTHORS file.
 
     This file is part of CZMQ, the high-level C binding for 0MQ:
@@ -27,7 +27,7 @@
 @header
     Expandable hash table container
 @discuss
-    Note that it's relatively slow (~50K insertions/deletes per second), so
+    Note that it's relatively slow (~50k insertions/deletes per second), so
     don't do inserts/updates on the critical path for message I/O. It can
     do ~2.5M lookups per second for 16-char keys. Timed on a 1.6GHz CPU.
 @end
@@ -466,7 +466,7 @@ zhash_foreach (zhash_t *self, zhash_foreach_fn *callback, void *argument)
 //  the file. If you use a null format, all comments are deleted.
 
 void
-zhash_comment (zhash_t *self, char *format, ...)
+zhash_comment (zhash_t *self, const char *format, ...)
 {
     if (format) {
         if (!self->comments) {
@@ -491,7 +491,7 @@ zhash_comment (zhash_t *self, char *format, ...)
 //  Returns 0 if OK, else -1 if a file error occurred
 
 int
-zhash_save (zhash_t *self, char *filename)
+zhash_save (zhash_t *self, const char *filename)
 {
     assert (self);
 
@@ -526,7 +526,7 @@ zhash_save (zhash_t *self, char *filename)
 //  Returns 0 if OK, else -1 if a file was not readable.
 
 int
-zhash_load (zhash_t *self, char *filename)
+zhash_load (zhash_t *self, const char *filename)
 {
     assert (self);
     zhash_autofree (self);
@@ -535,11 +535,11 @@ zhash_load (zhash_t *self, char *filename)
     //  modification date (0 for unknown files), so that zhash_refresh ()
     //  will always work after zhash_load (), to load a newly-created
     //  file.
-    
+
     //  Take copy of filename in case self->filename is same string.
-    filename = strdup (filename);
+    char *filename_copy = strdup (filename);
     free (self->filename);
-    self->filename = filename;
+    self->filename = filename_copy;
     self->modified = zsys_file_modified (self->filename);
     FILE *handle = fopen (self->filename, "r");
     if (!handle)
@@ -568,14 +568,14 @@ zhash_load (zhash_t *self, char *filename)
 //  --------------------------------------------------------------------------
 //  When a hash table was loaded from a file by zhash_load, this method will
 //  reload the file if it has been modified since, and is "stable", i.e. not
-//  still changing. Returns 0 if OK, -1 if there was an error reloading the 
+//  still changing. Returns 0 if OK, -1 if there was an error reloading the
 //  file.
 
 int
 zhash_refresh (zhash_t *self)
 {
     assert (self);
-    
+
     if (self->filename) {
         if (zsys_file_modified (self->filename) > self->modified
         &&  zsys_file_stable (self->filename)) {
@@ -608,6 +608,85 @@ zhash_autofree (zhash_t *self)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Serialize hash table to a binary frame that can be sent in a message.
+//  The packed format is compatible with the 'dictionary' type defined in
+//  rfc.zeromq.org/spec:19/FILEMQ. Comments are not included in the packed
+//  data. Item values MUST be strings.
+
+zframe_t *
+zhash_pack (zhash_t *self)
+{
+    assert (self);
+
+    //  First, calculate packed data size
+    size_t frame_size = 1;      //  Dictionary size, items
+    uint index;
+    for (index = 0; index != self->limit; index++) {
+        item_t *item = self->items [index];
+        while (item) {
+            //  [Length]key=value
+            frame_size += 2 + strlen (item->key) + strlen ((char *) item->value);
+            item = item->next;
+        }
+    }
+    //  Now serialize items into the frame
+    zframe_t *frame = zframe_new (NULL, frame_size);
+    byte *needle = zframe_data (frame);
+    *needle++ = (byte) self->size;
+    for (index = 0; index != self->limit; index++) {
+        item_t *item = self->items [index];
+        while (item) {
+            char string [256];
+            snprintf (string, 256, "%s=%s", item->key, (char *) item->value);
+            *needle++ = (byte) strlen (string);
+            memcpy (needle, string, strlen (string));
+            needle += strlen (string);
+            item = item->next;
+        }
+    }
+    return frame;
+}
+
+
+//  --------------------------------------------------------------------------
+//  Unpack binary frame into a new hash table. Packed data must follow format
+//  defined by zhash_pack. Hash table is set to autofree. An empty frame
+//  unpacks to an empty hash table.
+
+zhash_t *
+zhash_unpack (zframe_t *frame)
+{
+    zhash_t *self = zhash_new ();
+    assert (self);
+    zhash_autofree (self);
+
+    assert (frame);
+    if (zframe_size (frame) < 1)
+        return self;            //  Arguable...
+
+    byte *needle = zframe_data (frame);
+    byte *ceiling = needle + zframe_size (frame);
+    size_t nbr_items = *needle++;
+    while (nbr_items && needle < ceiling) {
+        size_t string_size = *needle++;
+        if (needle + string_size <= ceiling) {
+            char string [256];
+            memcpy (string, needle, string_size);
+            string [string_size] = 0;
+            needle += string_size;
+            char *value = strchr (string, '=');
+            if (value) {
+                *value++ = 0;
+                //  Takes copy of value automatically
+                zhash_insert (self, string, value);
+            }
+        }
+    }
+    return self;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Runs selftest of class
 //
@@ -707,6 +786,16 @@ zhash_test (int verbose)
     assert (streq (item, "dead beef"));
     zhash_destroy (&copy);
 
+    //  Test pack/unpack methods
+    zframe_t *frame = zhash_pack (hash);
+    copy = zhash_unpack (frame);
+    zframe_destroy (&frame);
+    assert (zhash_size (copy) == 4);
+    item = (char *) zhash_lookup (copy, "LIVEBEEF");
+    assert (item);
+    assert (streq (item, "dead beef"));
+    zhash_destroy (&copy);
+
     // Test foreach
     assert (0 == zhash_foreach (hash, test_foreach, hash));
     assert (-1 == zhash_foreach (hash, test_foreach_error, hash));
