@@ -2,7 +2,7 @@
     zstr - sending and receiving strings
 
     -------------------------------------------------------------------------
-    Copyright (c) 1991-2013 iMatix Corporation <www.imatix.com>
+    Copyright (c) 1991-2014 iMatix Corporation <www.imatix.com>
     Copyright other contributors as noted in the AUTHORS file.
 
     This file is part of CZMQ, the high-level C binding for 0MQ:
@@ -36,31 +36,32 @@
 #include "../include/czmq.h"
 
 static int
-s_send_string (void *zocket, bool more, char *string)
+s_send_string (void *socket, bool more, char *string)
 {
-    assert (zocket);
+    assert (socket);
 
     int len = strlen (string);
     zmq_msg_t message;
     zmq_msg_init_size (&message, len);
     memcpy (zmq_msg_data (&message), string, len);
-    int rc = zmq_sendmsg (zocket, &message, more? ZMQ_SNDMORE: 0);
+    int rc = zmq_sendmsg (socket, &message, more? ZMQ_SNDMORE: 0);
 
     return rc == -1? -1: 0;
 }
 
 
 //  --------------------------------------------------------------------------
-//  Receive C string from socket. Caller must free returned string. Returns
-//  NULL if the context is being terminated or the process was interrupted.
+//  Receive C string from socket. Caller must free returned string using
+//  zstr_free(). Returns NULL if the context is being terminated or the
+//  process was interrupted.
 
 char *
-zstr_recv (void *zocket)
+zstr_recv (void *socket)
 {
-    assert (zocket);
+    assert (socket);
     zmq_msg_t message;
     zmq_msg_init (&message);
-    if (zmq_recvmsg (zocket, &message, 0) < 0)
+    if (zmq_recvmsg (socket, &message, 0) < 0)
         return NULL;
 
     size_t size = zmq_msg_size (&message);
@@ -74,17 +75,17 @@ zstr_recv (void *zocket)
 
 //  --------------------------------------------------------------------------
 //  Receive C string from socket, if socket had input ready. Caller must
-//  free returned string. Returns NULL if there was no input waiting, or if
-//  the context was terminated. Use zctx_interrupted to exit any loop that
-//  relies on this method.
+//  free returned string using zstr_free. Returns NULL if there was no input
+//  waiting, or if the context was terminated. Use zctx_interrupted to exit
+//  any loop that relies on this method.
 
 char *
-zstr_recv_nowait (void *zocket)
+zstr_recv_nowait (void *socket)
 {
-    assert (zocket);
+    assert (socket);
     zmq_msg_t message;
     zmq_msg_init (&message);
-    if (zmq_recvmsg (zocket, &message, ZMQ_DONTWAIT) < 0)
+    if (zmq_recvmsg (socket, &message, ZMQ_DONTWAIT) < 0)
         return NULL;
 
     size_t size = zmq_msg_size (&message);
@@ -97,41 +98,71 @@ zstr_recv_nowait (void *zocket)
 
 
 //  --------------------------------------------------------------------------
-//  Send C string to socket. Returns 0 if successful, -1 if there was an
-//  error.
+//  Send a C string to a socket, as a frame. The string is sent without
+//  trailing null byte; to read this you can use zstr_recv, or a similar
+//  method that adds a null terminator on the received string.
 
 int
-zstr_send (void *zocket, const char *format, ...)
+zstr_send (void *socket, const char *string)
 {
-    assert (zocket);
+    assert (socket);
+    assert (string);
+    return s_send_string (socket, false, (char *) string);
+}
+
+
+//  --------------------------------------------------------------------------
+//  Send a C string to a socket, as zstr_send(), with a MORE flag, so that
+//  you can send further strings in the same multi-part message.
+
+int
+zstr_sendm (void *socket, const char *string)
+{
+    assert (socket);
+    assert (string);
+    return s_send_string (socket, true, (char *) string);
+}
+
+
+//  --------------------------------------------------------------------------
+//  Send a formatted string to a socket. Note that you should NOT use
+//  user-supplied strings in the format (they may contain '%' which
+//  will create security holes).
+
+int
+zstr_sendf (void *socket, const char *format, ...)
+{
+    assert (socket);
     assert (format);
-    
+
     va_list argptr;
     va_start (argptr, format);
     char *string = zsys_vprintf (format, argptr);
     va_end (argptr);
-    
-    int rc = s_send_string (zocket, false, string);
+
+    int rc = s_send_string (socket, false, string);
     free (string);
     return rc;
 }
 
 
 //  --------------------------------------------------------------------------
-//  Send a string to a socket in 0MQ string format, with MORE flag
+//  Send a formatted string to a socket, as for zstr_sendf(), with a
+//  MORE flag, so that you can send further strings in the same multi-part
+//  message.
 
 int
-zstr_sendm (void *zocket, const char *format, ...)
+zstr_sendfm (void *socket, const char *format, ...)
 {
-    assert (zocket);
+    assert (socket);
     assert (format);
-    
+
     va_list argptr;
     va_start (argptr, format);
     char *string = zsys_vprintf (format, argptr);
     va_end (argptr);
-    
-    int rc = s_send_string (zocket, true, string);
+
+    int rc = s_send_string (socket, true, string);
     free (string);
     return rc;
 }
@@ -157,11 +188,12 @@ zstr_sendx (void *socket, const char *string, ...)
 
 
 //  --------------------------------------------------------------------------
-//  Receive a series of strings (until NULL) from multipart data
+//  Receive a series of strings (until NULL) from multipart data.
 //  Each string is allocated and filled with string data; if there
 //  are not enough frames, unallocated strings are set to NULL.
 //  Returns -1 if the message could not be read, else returns the
-//  number of strings filled, zero or more.
+//  number of strings filled, zero or more. Free each returned string
+//  using zstr_free().
 
 int
 zstr_recvx (void *socket, char **string_p, ...)
@@ -182,6 +214,19 @@ zstr_recvx (void *socket, char **string_p, ...)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Free a provided string, and nullify the parent pointer. Safe to call on
+//  a null pointer.
+
+void
+zstr_free (char **string_p)
+{
+    assert (string_p);
+    free (*string_p);
+    *string_p = NULL;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Selftest
 
@@ -204,7 +249,7 @@ zstr_test (bool verbose)
     //  Send ten strings, five strings with MORE flag and then END
     int string_nbr;
     for (string_nbr = 0; string_nbr < 10; string_nbr++)
-        zstr_send (output, "this is string %d", string_nbr);
+        zstr_sendf (output, "this is string %d", string_nbr);
     zstr_sendx (output, "This", "is", "almost", "the", "very", "END", NULL);
 
     //  Read and count until we receive END
@@ -212,10 +257,10 @@ zstr_test (bool verbose)
     for (string_nbr = 0;; string_nbr++) {
         char *string = zstr_recv (input);
         if (streq (string, "END")) {
-            free (string);
+            zstr_free (&string);
             break;
         }
-        free (string);
+        zstr_free (&string);
     }
     assert (string_nbr == 15);
 
